@@ -1,19 +1,22 @@
-
-import os
-import errno
 import atexit
-import operator
-import six
-import sys
-from functools import reduce
-import uuid
-from six.moves import map
-import itertools
-import shutil
-from os.path import expanduser, expandvars, isabs, sep
-from posixpath import sep as posix_sep
+import errno
 import fnmatch
+import itertools
+import operator
+import os
+import shutil
 import stat
+import sys
+import uuid
+from functools import reduce
+from os.path import expanduser
+from os.path import expandvars
+from os.path import isabs
+from os.path import sep
+from posixpath import sep as posix_sep
+
+import six
+from six.moves import map
 
 from .compat import PY36
 
@@ -156,7 +159,7 @@ def create_cleanup_lock(p):
     else:
         pid = os.getpid()
         spid = str(pid)
-        if not isinstance(spid, six.binary_type):
+        if not isinstance(spid, bytes):
             spid = spid.encode("ascii")
         os.write(fd, spid)
         os.close(fd)
@@ -182,9 +185,15 @@ def register_cleanup_lock_removal(lock_path, register=atexit.register):
     return register(cleanup_on_exit)
 
 
-def delete_a_numbered_dir(path):
-    """removes a numbered directory"""
-    create_cleanup_lock(path)
+def maybe_delete_a_numbered_dir(path):
+    """removes a numbered directory if its lock can be obtained"""
+    try:
+        create_cleanup_lock(path)
+    except (OSError, EnvironmentError):
+        #  known races:
+        #  * other process did a cleanup at the same time
+        #  * deletable folder was found
+        return
     parent = path.parent
 
     garbage = parent.joinpath("garbage-{}".format(uuid.uuid4()))
@@ -214,7 +223,7 @@ def ensure_deletable(path, consider_lock_dead_if_created_before):
 def try_cleanup(path, consider_lock_dead_if_created_before):
     """tries to cleanup a folder if we can ensure its deletable"""
     if ensure_deletable(path, consider_lock_dead_if_created_before):
-        delete_a_numbered_dir(path)
+        maybe_delete_a_numbered_dir(path)
 
 
 def cleanup_candidates(root, prefix, keep):
@@ -245,8 +254,8 @@ def make_numbered_dir_with_cleanup(root, prefix, keep, lock_timeout):
             p = make_numbered_dir(root, prefix)
             lock_path = create_cleanup_lock(p)
             register_cleanup_lock_removal(lock_path)
-        except Exception as e:
-            pass
+        except Exception as exc:
+            e = exc
         else:
             consider_lock_dead_if_created_before = p.stat().st_mtime - lock_timeout
             cleanup_numbered_dir(
